@@ -19,6 +19,7 @@
 package org.apache.commons.compress.archivers.cpio;
 
 import java.io.File;
+import java.util.Date;
 
 import org.apache.commons.compress.archivers.ArchiveEntry;
 
@@ -81,7 +82,7 @@ import org.apache.commons.compress.archivers.ArchiveEntry;
  * c_filesize[2]    Length of the file in bytes. This is the length of the data 
  *                  section that follows the header structure. Must be 0 for 
  *                  FIFOs and directories
- *               
+ *
  * All fields are unsigned short fields with 16-bit integer values
  * apart from c_mtime and c_filesize which are 32-bit integer values
  * </pre>
@@ -145,9 +146,9 @@ import org.apache.commons.compress.archivers.ArchiveEntry;
 public class CpioArchiveEntry implements CpioConstants, ArchiveEntry {
 
     // Header description fields - should be same throughout an archive
-    
+
     /**
-     * See {@link CpioArchiveEntry#setFormat(short)} for possible values.
+     * See constructor documenation for possible values.
      */
     private final short fileFormat; 
 
@@ -158,7 +159,7 @@ public class CpioArchiveEntry implements CpioConstants, ArchiveEntry {
     private final int alignmentBoundary;
 
     // Header fields
-    
+
     private long chksum = 0;
 
     /** Number of bytes in the file */
@@ -187,7 +188,7 @@ public class CpioArchiveEntry implements CpioConstants, ArchiveEntry {
     private long uid = 0;
 
     /**
-     * Ceates a CPIOArchiveEntry with a specified format.
+     * Creates a CPIOArchiveEntry with a specified format.
      * 
      * @param format
      *            The cpio format for this entry.
@@ -225,20 +226,41 @@ public class CpioArchiveEntry implements CpioConstants, ArchiveEntry {
     }
 
     /**
-     * Ceates a CPIOArchiveEntry with a specified name. The format of this entry
-     * will be the new format.
+     * Creates a CPIOArchiveEntry with a specified name. The format of
+     * this entry will be the new format.
      * 
      * @param name
      *            The name of this entry.
      */
     public CpioArchiveEntry(final String name) {
-        this(FORMAT_NEW);
+        this(FORMAT_NEW, name);
+    }
+
+    /**
+     * Creates a CPIOArchiveEntry with a specified name.
+     * 
+     * @param format
+     *            The cpio format for this entry.
+     * @param name
+     *            The name of this entry.
+     * <br/>
+     * Possible format values are:
+     * <p>
+     * CpioConstants.FORMAT_NEW<br/>
+     * CpioConstants.FORMAT_NEW_CRC<br/>
+     * CpioConstants.FORMAT_OLD_BINARY<br/>
+     * CpioConstants.FORMAT_OLD_ASCII<br/>
+     * 
+     * @since 1.1
+     */
+    public CpioArchiveEntry(final short format, final String name) {
+        this(format);
         this.name = name;
     }
 
     /**
-     * Creates a CPIOArchiveEntry with a specified name. The format of this entry
-     * will be the new format.
+     * Creates a CPIOArchiveEntry with a specified name. The format of
+     * this entry will be the new format.
      * 
      * @param name
      *            The name of this entry.
@@ -246,23 +268,84 @@ public class CpioArchiveEntry implements CpioConstants, ArchiveEntry {
      *            The size of this entry
      */
     public CpioArchiveEntry(final String name, final long size) {
-        this(FORMAT_NEW);
-        this.name = name;
+        this(name);
         this.setSize(size);
     }
 
+    /**
+     * Creates a CPIOArchiveEntry with a specified name.
+     * 
+     * @param format
+     *            The cpio format for this entry.
+     * @param name
+     *            The name of this entry.
+     * @param size
+     *            The size of this entry
+     * <br/>
+     * Possible format values are:
+     * <p>
+     * CpioConstants.FORMAT_NEW<br/>
+     * CpioConstants.FORMAT_NEW_CRC<br/>
+     * CpioConstants.FORMAT_OLD_BINARY<br/>
+     * CpioConstants.FORMAT_OLD_ASCII<br/>
+     * 
+     * @since 1.1
+     */
+    public CpioArchiveEntry(final short format, final String name,
+                            final long size) {
+        this(format, name);
+        this.setSize(size);
+    }
+
+    /**
+     * Creates a CPIOArchiveEntry with a specified name for a
+     * specified file. The format of this entry will be the new
+     * format.
+     * 
+     * @param inputFile
+     *            The file to gather information from.
+     * @param entryName
+     *            The name of this entry.
+     */
     public CpioArchiveEntry(File inputFile, String entryName) {
-        this(entryName, inputFile.isFile() ? inputFile.length() : 0);
+        this(FORMAT_NEW, inputFile, entryName);
+    }
+
+    /**
+     * Creates a CPIOArchiveEntry with a specified name for a
+     * specified file.
+     * 
+     * @param format
+     *            The cpio format for this entry.
+     * @param inputFile
+     *            The file to gather information from.
+     * @param entryName
+     *            The name of this entry.
+     * <br/>
+     * Possible format values are:
+     * <p>
+     * CpioConstants.FORMAT_NEW<br/>
+     * CpioConstants.FORMAT_NEW_CRC<br/>
+     * CpioConstants.FORMAT_OLD_BINARY<br/>
+     * CpioConstants.FORMAT_OLD_ASCII<br/>
+     * 
+     * @since 1.1
+     */
+    public CpioArchiveEntry(final short format, File inputFile,
+                            String entryName) {
+        this(format, entryName, inputFile.isFile() ? inputFile.length() : 0);
         long mode=0;
         if (inputFile.isDirectory()){
             mode |= C_ISDIR;
         } else if (inputFile.isFile()){
             mode |= C_ISREG;
         } else {
-            throw new IllegalArgumentException("Cannot determine type of file "+inputFile.getName());
+            throw new IllegalArgumentException("Cannot determine type of file "
+                                               + inputFile.getName());
         }
         // TODO set other fields as needed
         setMode(mode);
+        setTime(inputFile.lastModified() / 1000);
     }
 
     /**
@@ -384,7 +467,7 @@ public class CpioArchiveEntry implements CpioConstants, ArchiveEntry {
      * @return the number of bytes needed to pad the header (0,1,2,3)
      */
     public int getHeaderPadCount(){
-        if (this.alignmentBoundary == 0) return 0;
+        if (this.alignmentBoundary == 0) { return 0; }
         int size = this.headerSize+this.name.length()+1; // Name has terminating null
         int remain = size % this.alignmentBoundary;
         if (remain > 0){
@@ -399,7 +482,7 @@ public class CpioArchiveEntry implements CpioConstants, ArchiveEntry {
      * @return the number of bytes needed to pad the data (0,1,2,3)
      */
     public int getDataPadCount(){
-        if (this.alignmentBoundary == 0) return 0;
+        if (this.alignmentBoundary == 0) { return 0; }
         long size = this.filesize;
         int remain = (int) (size % this.alignmentBoundary);
         if (remain > 0){
@@ -423,7 +506,7 @@ public class CpioArchiveEntry implements CpioConstants, ArchiveEntry {
      * @return Returns the mode.
      */
     public long getMode() {
-        return this.mode;
+        return mode == 0 && !CPIO_TRAILER.equals(name) ? C_ISREG : mode;
     }
 
     /**
@@ -441,7 +524,9 @@ public class CpioArchiveEntry implements CpioConstants, ArchiveEntry {
      * @return Returns the number of links.
      */
     public long getNumberOfLinks() {
-        return this.nlink;
+        return nlink == 0 ?
+            (isDirectory() ? 2 : 1)
+            : nlink;
     }
 
     /**
@@ -492,6 +577,11 @@ public class CpioArchiveEntry implements CpioConstants, ArchiveEntry {
         return this.mtime;
     }
 
+    /** {@inheritDoc} */
+    public Date getLastModifiedDate() {
+        return new Date(1000 * getTime());
+    }
+
     /**
      * Get the user id.
      * 
@@ -630,7 +720,7 @@ public class CpioArchiveEntry implements CpioConstants, ArchiveEntry {
     public void setSize(final long size) {
         if (size < 0 || size > 0xFFFFFFFFL) {
             throw new IllegalArgumentException("invalid entry size <" + size
-                    + ">");
+                                               + ">");
         }
         this.filesize = size;
     }
@@ -675,9 +765,9 @@ public class CpioArchiveEntry implements CpioConstants, ArchiveEntry {
             break;
         default:
             throw new IllegalArgumentException(
-                    "Unknown mode. "
-                    + "Full: " + Long.toHexString(mode) 
-                    + " Masked: " + Long.toHexString(maskedMode));
+                                               "Unknown mode. "
+                                               + "Full: " + Long.toHexString(mode) 
+                                               + " Masked: " + Long.toHexString(maskedMode));
         }
 
         this.mode = mode;
@@ -768,6 +858,7 @@ public class CpioArchiveEntry implements CpioConstants, ArchiveEntry {
     /* (non-Javadoc)
      * @see java.lang.Object#hashCode()
      */
+    @Override
     public int hashCode() {
         final int prime = 31;
         int result = 1;
@@ -778,6 +869,7 @@ public class CpioArchiveEntry implements CpioConstants, ArchiveEntry {
     /* (non-Javadoc)
      * @see java.lang.Object#equals(java.lang.Object)
      */
+    @Override
     public boolean equals(Object obj) {
         if (this == obj) {
             return true;
