@@ -23,18 +23,19 @@
 
 package org.apache.commons.compress.archivers.tar;
 
-import java.io.BufferedReader;
+import java.io.ByteArrayOutputStream;
 import java.io.IOException;
 import java.io.InputStream;
-import java.io.InputStreamReader;
 import java.util.HashMap;
-import java.util.Iterator;
 import java.util.Map;
 import java.util.Map.Entry;
 
 import org.apache.commons.compress.archivers.ArchiveEntry;
 import org.apache.commons.compress.archivers.ArchiveInputStream;
+import org.apache.commons.compress.archivers.zip.ZipEncoding;
+import org.apache.commons.compress.archivers.zip.ZipEncodingHelper;
 import org.apache.commons.compress.utils.ArchiveUtils;
+import org.apache.commons.compress.utils.CharsetNames;
 
 /**
  * The TarInputStream reads a UNIX tar archive as an InputStream.
@@ -53,6 +54,7 @@ public class TarArchiveInputStream extends ArchiveInputStream {
     private byte[] readBuf;
     protected final TarBuffer buffer;
     private TarArchiveEntry currEntry;
+    private final ZipEncoding encoding;
 
     /**
      * Constructor for TarInputStream.
@@ -62,6 +64,16 @@ public class TarArchiveInputStream extends ArchiveInputStream {
         this(is, TarBuffer.DEFAULT_BLKSIZE, TarBuffer.DEFAULT_RCDSIZE);
     }
 
+    /**
+     * Constructor for TarInputStream.
+     * @param is the input stream to use
+     * @param encoding name of the encoding to use for file names
+     * @since Commons Compress 1.4
+     */
+    public TarArchiveInputStream(InputStream is, String encoding) {
+        this(is, TarBuffer.DEFAULT_BLKSIZE, TarBuffer.DEFAULT_RCDSIZE, encoding);
+    }
+
     /**
      * Constructor for TarInputStream.
      * @param is the input stream to use
@@ -71,6 +83,18 @@ public class TarArchiveInputStream extends ArchiveInputStream {
         this(is, blockSize, TarBuffer.DEFAULT_RCDSIZE);
     }
 
+    /**
+     * Constructor for TarInputStream.
+     * @param is the input stream to use
+     * @param blockSize the block size to use
+     * @param encoding name of the encoding to use for file names
+     * @since Commons Compress 1.4
+     */
+    public TarArchiveInputStream(InputStream is, int blockSize,
+                                 String encoding) {
+        this(is, blockSize, TarBuffer.DEFAULT_RCDSIZE, encoding);
+    }
+
     /**
      * Constructor for TarInputStream.
      * @param is the input stream to use
@@ -78,15 +102,30 @@ public class TarArchiveInputStream extends ArchiveInputStream {
      * @param recordSize the record size to use
      */
     public TarArchiveInputStream(InputStream is, int blockSize, int recordSize) {
+        this(is, blockSize, recordSize, null);
+    }
+
+    /**
+     * Constructor for TarInputStream.
+     * @param is the input stream to use
+     * @param blockSize the block size to use
+     * @param recordSize the record size to use
+     * @param encoding name of the encoding to use for file names
+     * @since Commons Compress 1.4
+     */
+    public TarArchiveInputStream(InputStream is, int blockSize, int recordSize,
+                                 String encoding) {
         this.buffer = new TarBuffer(is, blockSize, recordSize);
         this.readBuf = null;
         this.hasHitEOF = false;
+        this.encoding = ZipEncodingHelper.getZipEncoding(encoding);
     }
 
     /**
      * Closes this stream. Calls the TarBuffer's close() method.
      * @throws IOException on error
      */
+    @Override
     public void close() throws IOException {
         buffer.close();
     }
@@ -112,6 +151,7 @@ public class TarArchiveInputStream extends ArchiveInputStream {
      * @return The number of available bytes for the current entry.
      * @throws IOException for signature
      */
+    @Override
     public int available() throws IOException {
         if (entrySize - entryOffset > Integer.MAX_VALUE) {
             return Integer.MAX_VALUE;
@@ -129,6 +169,7 @@ public class TarArchiveInputStream extends ArchiveInputStream {
      * @return the number actually skipped
      * @throws IOException on error
      */
+    @Override
     public long skip(long numToSkip) throws IOException {
         // REVIEW
         // This is horribly inefficient, but it ensures that we
@@ -150,6 +191,7 @@ public class TarArchiveInputStream extends ArchiveInputStream {
     /**
      * Since we do not support marking just yet, we do nothing.
      */
+    @Override
     public synchronized void reset() {
     }
 
@@ -192,7 +234,13 @@ public class TarArchiveInputStream extends ArchiveInputStream {
             return null;
         }
 
-        currEntry = new TarArchiveEntry(headerBuf);
+        try {
+            currEntry = new TarArchiveEntry(headerBuf, encoding);
+        } catch (IllegalArgumentException e) {
+            IOException ioe = new IOException("Error detected parsing the header");
+            ioe.initCause(e);
+            throw ioe;
+        }
         entryOffset = 0;
         entrySize = currEntry.getSize();
 
@@ -202,7 +250,7 @@ public class TarArchiveInputStream extends ArchiveInputStream {
             byte[] buf = new byte[SMALL_BUFFER_SIZE];
             int length = 0;
             while ((length = read(buf)) >= 0) {
-                longName.append(new String(buf, 0, length));
+                longName.append(new String(buf, 0, length)); // TODO default charset?
             }
             getNextEntry();
             if (currEntry == null) {
@@ -226,6 +274,11 @@ public class TarArchiveInputStream extends ArchiveInputStream {
             readGNUSparse();
         }
 
+        // If the size of the next element in the archive has changed
+        // due to a new size being reported in the posix header
+        // information, we update entrySize here so that it contains
+        // the correct value.
+        entrySize = currEntry.getSize();
         return currEntry;
     }
 
@@ -258,33 +311,44 @@ public class TarArchiveInputStream extends ArchiveInputStream {
     }
 
     private void paxHeaders() throws IOException{
-        BufferedReader br = new BufferedReader(new InputStreamReader(this, "UTF-8"));
-        Map headers = new HashMap();
+        Map<String, String> headers = parsePaxHeaders(this);
+        getNextEntry(); // Get the actual file entry
+        applyPaxHeadersToCurrentEntry(headers);
+    }
+
+    Map<String, String> parsePaxHeaders(InputStream i) throws IOException {
+        Map<String, String> headers = new HashMap<String, String>();
         // Format is "length keyword=value\n";
         while(true){ // get length
             int ch;
-            int len=0;
-            int read=0;
-            while((ch = br.read()) != -1){
+            int len = 0;
+            int read = 0;
+            while((ch = i.read()) != -1) {
                 read++;
                 if (ch == ' '){ // End of length string
                     // Get keyword
-                    StringBuffer sb = new StringBuffer();
-                    while((ch = br.read()) != -1){
+                    ByteArrayOutputStream coll = new ByteArrayOutputStream();
+                    while((ch = i.read()) != -1) {
                         read++;
                         if (ch == '='){ // end of keyword
-                            String keyword = sb.toString();
+                            String keyword = coll.toString(CharsetNames.UTF_8);
                             // Get rest of entry
-                            char[] cbuf = new char[len-read];
-                            int got = br.read(cbuf);
-                            if (got != len-read){
-                                throw new IOException("Failed to read Paxheader. Expected "+(len-read)+" chars, read "+got);
+                            byte[] rest = new byte[len - read];
+                            int got = i.read(rest);
+                            if (got != len - read){
+                                throw new IOException("Failed to read "
+                                                      + "Paxheader. Expected "
+                                                      + (len - read)
+                                                      + " bytes, read "
+                                                      + got);
                             }
-                            String value = new String(cbuf, 0 , len-read-1); // Drop trailing NL
+                            // Drop trailing NL
+                            String value = new String(rest, 0,
+                                                      len - read - 1, CharsetNames.UTF_8);
                             headers.put(keyword, value);
                             break;
                         }
-                        sb.append((char)ch);
+                        coll.write((byte) ch);
                     }
                     break; // Processed single header
                 }
@@ -295,21 +359,24 @@ public class TarArchiveInputStream extends ArchiveInputStream {
                 break;
             }
         }
-        getNextEntry(); // Get the actual file entry
+        return headers;
+    }
+
+    private void applyPaxHeadersToCurrentEntry(Map<String, String> headers) {
         /*
          * The following headers are defined for Pax.
-         * atime, ctime, mtime, charset: cannot use these without changing TarArchiveEntry fields
+         * atime, ctime, charset: cannot use these without changing TarArchiveEntry fields
+         * mtime
          * comment
          * gid, gname
          * linkpath
          * size
          * uid,uname
+         * SCHILY.devminor, SCHILY.devmajor: don't have setters/getters for those
          */
-        Iterator hdrs = headers.entrySet().iterator();
-        while(hdrs.hasNext()){
-            Entry ent = (Entry) hdrs.next();
-            String key = (String) ent.getKey();
-            String val = (String) ent.getValue();
+        for (Entry<String, String> ent : headers.entrySet()){
+            String key = ent.getKey();
+            String val = ent.getValue();
             if ("path".equals(key)){
                 currEntry.setName(val);
             } else if ("linkpath".equals(key)){
@@ -324,6 +391,12 @@ public class TarArchiveInputStream extends ArchiveInputStream {
                 currEntry.setUserName(val);
             } else if ("size".equals(key)){
                 currEntry.setSize(Long.parseLong(val));
+            } else if ("mtime".equals(key)){
+                currEntry.setModTime((long) (Double.parseDouble(val) * 1000));
+            } else if ("SCHILY.devminor".equals(key)){
+                currEntry.setDevMinor(Integer.parseInt(val));
+            } else if ("SCHILY.devmajor".equals(key)){
+                currEntry.setDevMajor(Integer.parseInt(val));
             }
         }
     }
@@ -357,6 +430,7 @@ public class TarArchiveInputStream extends ArchiveInputStream {
         }
     }
 
+    @Override
     public ArchiveEntry getNextEntry() throws IOException {
         return getNextTarEntry();
     }
@@ -374,6 +448,7 @@ public class TarArchiveInputStream extends ArchiveInputStream {
      * @return The number of bytes read, or -1 at EOF.
      * @throws IOException on error
      */
+    @Override
     public int read(byte[] buf, int offset, int numToRead) throws IOException {
         int totalRead = 0;
 
@@ -446,6 +521,7 @@ public class TarArchiveInputStream extends ArchiveInputStream {
      *
      * <p>May return false if the current entry is a sparse file.</p>
      */
+    @Override
     public boolean canReadEntryData(ArchiveEntry ae) {
         if (ae instanceof TarArchiveEntry) {
             TarArchiveEntry te = (TarArchiveEntry) ae;
