@@ -27,8 +27,8 @@ import java.io.IOException;
 import java.io.InputStream;
 import java.io.OutputStream;
 import java.util.ArrayList;
-import java.util.Iterator;
 import java.util.List;
+import java.util.Locale;
 
 import junit.framework.TestCase;
 
@@ -44,8 +44,8 @@ public abstract class AbstractTestCase extends TestCase {
     protected File resultDir;
 
     private File archive; // used to delete the archive in tearDown
-    protected List archiveList; // Lists the content of the archive as originally created
-    
+    protected List<String> archiveList; // Lists the content of the archive as originally created
+
     protected ArchiveStreamFactory factory = new ArchiveStreamFactory();
 
     public AbstractTestCase() {
@@ -56,6 +56,7 @@ public abstract class AbstractTestCase extends TestCase {
         super(name);
     }
 
+    @Override
     protected void setUp() throws Exception {
         dir = mkdir("dir");
         resultDir = mkdir("dir-result");
@@ -73,43 +74,64 @@ public abstract class AbstractTestCase extends TestCase {
         return new File("src/test/resources", path);
     }
 
+    @Override
     protected void tearDown() throws Exception {
         rmdir(dir);
         rmdir(resultDir);
         dir = resultDir = null;
-        if (archive != null && archive.exists()) {
-            if (!archive.delete()){
-                // Note: this exception won't be shown if the test has already failed
-                throw new Exception("Could not delete "+archive.getPath());
-            }
+        if (!tryHardToDelete(archive)) {
+            // Note: this exception won't be shown if the test has already failed
+            throw new Exception("Could not delete "+archive.getPath());
         }
     }
 
     protected static void rmdir(File f) {
-        // Sometimes fails without a pause - perhaps file close is partially asynchronous?
-        try {
-            Thread.sleep(20);
-        } catch (InterruptedException e) {
-        }
         String[] s = f.list();
         if (s != null) {
-            for (int i = 0; i < s.length; i++) {
-                final File file = new File(f, s[i]);
+            for (String element : s) {
+                final File file = new File(f, element);
                 if (file.isDirectory()){
                     rmdir(file);
                 }
-                boolean ok = file.delete();
+                boolean ok = tryHardToDelete(file);
                 if (!ok && file.exists()){
-                    System.out.println("Failed to delete "+s[i]+" in "+f.getPath());
+                    System.out.println("Failed to delete "+element+" in "+f.getPath());
                 }
             }
         }
-        f.delete(); // safer to delete and check
+        tryHardToDelete(f); // safer to delete and check
         if (f.exists()){
             throw new Error("Failed to delete "+f.getPath());
         }
     }
 
+    private static final boolean ON_WINDOWS =
+        System.getProperty("os.name").toLowerCase(Locale.ENGLISH)
+        .indexOf("windows") > -1;
+
+    /**
+     * Accommodate Windows bug encountered in both Sun and IBM JDKs.
+     * Others possible. If the delete does not work, call System.gc(),
+     * wait a little and try again.
+     *
+     * @return whether deletion was successful
+     * @since Stolen from FileUtils in Ant 1.8.0
+     */
+    public static boolean tryHardToDelete(File f) {
+        if (f != null && f.exists() && !f.delete()) {
+            if (ON_WINDOWS) {
+                System.gc();
+            }
+            try {
+                Thread.sleep(10);
+            } catch (InterruptedException ex) {
+                // Ignore Exception
+            }
+            return f.delete();
+        }
+        return true;
+    }
+
     /**
      * Creates an archive of textbased files in several directories. The
      * archivername is the factory identifier for the archiver, for example zip,
@@ -139,7 +161,8 @@ public abstract class AbstractTestCase extends TestCase {
         OutputStream stream = null;
         try {
             archive = File.createTempFile("test", "." + archivename);
-            archiveList = new ArrayList();
+            archive.deleteOnExit();
+            archiveList = new ArrayList<String>();
 
             stream = new FileOutputStream(archive);
             out = factory.createArchiveOutputStream(archivename, stream);
@@ -198,9 +221,10 @@ public abstract class AbstractTestCase extends TestCase {
     protected File createEmptyArchive(String archivename) throws Exception {
         ArchiveOutputStream out = null;
         OutputStream stream = null;
-        archiveList = new ArrayList();
+        archiveList = new ArrayList<String>();
         try {
             archive = File.createTempFile("empty", "." + archivename);
+            archive.deleteOnExit();
             stream = new FileOutputStream(archive);
             out = factory.createArchiveOutputStream(archivename, stream);
             out.finish();
@@ -224,9 +248,10 @@ public abstract class AbstractTestCase extends TestCase {
     protected File createSingleEntryArchive(String archivename) throws Exception {
         ArchiveOutputStream out = null;
         OutputStream stream = null;
-        archiveList = new ArrayList();
+        archiveList = new ArrayList<String>();
         try {
             archive = File.createTempFile("empty", "." + archivename);
+            archive.deleteOnExit();
             stream = new FileOutputStream(archive);
             out = factory.createArchiveOutputStream(archivename, stream);
             // Use short file name so does not cause problems for ar
@@ -251,7 +276,7 @@ public abstract class AbstractTestCase extends TestCase {
      *            a list with expected string filenames
      * @throws Exception
      */
-    protected void checkArchiveContent(File archive, List expected)
+    protected void checkArchiveContent(File archive, List<String> expected)
             throws Exception {
         final InputStream is = new FileInputStream(archive);
         try {
@@ -267,28 +292,27 @@ public abstract class AbstractTestCase extends TestCase {
      * Checks that an archive input stream can be read, and that the file data matches file sizes.
      * 
      * @param in
-     * @param expected list of expected entries or <code>null</code> if no check of names desired
+     * @param expected list of expected entries or {@code null} if no check of names desired
      * @throws Exception
      */
-    protected void checkArchiveContent(ArchiveInputStream in, List expected)
+    protected void checkArchiveContent(ArchiveInputStream in, List<String> expected)
             throws Exception {
         checkArchiveContent(in, expected, true);
     }
-    
+
     /**
      * Checks that an archive input stream can be read, and that the file data matches file sizes.
      * 
      * @param in
-     * @param expected list of expected entries or <code>null</code> if no check of names desired
+     * @param expected list of expected entries or {@code null} if no check of names desired
      * @param cleanUp Cleans up resources if true 
      * @return returns the created result file if cleanUp = false, or null otherwise 
      * @throws Exception
      */
-    protected File checkArchiveContent(ArchiveInputStream in, List expected, boolean cleanUp)
+    protected File checkArchiveContent(ArchiveInputStream in, List<String> expected, boolean cleanUp)
             throws Exception {
-        File result = File.createTempFile("dir-result", "");
-        result.delete();
-        result.mkdir();
+        File result = mkdir("dir-result");
+        result.deleteOnExit();
 
         try {
             ArchiveEntry entry = null;
@@ -305,7 +329,7 @@ public abstract class AbstractTestCase extends TestCase {
                         copied=IOUtils.copy(in, out);
                     } finally {
                         out.close();
-                    }                    
+                    }
                 }
                 final long size = entry.getSize();
                 if (size != ArchiveEntry.SIZE_UNKNOWN) {
@@ -321,8 +345,7 @@ public abstract class AbstractTestCase extends TestCase {
             }
             in.close();
             if (expected != null && expected.size() > 0) {
-                for (Iterator iterator = expected.iterator(); iterator.hasNext();) {
-                    String name = (String) iterator.next();
+                for (String name : expected) {
                     fail("Expected entry: " + name);
                 }
             }
@@ -354,9 +377,7 @@ public abstract class AbstractTestCase extends TestCase {
      * element of the two element array).
      */
     protected File[] createTempDirAndFile() throws IOException {
-        File tmpDir = File.createTempFile("testdir", "");
-        tmpDir.delete();
-        tmpDir.mkdir();
+        File tmpDir = mkdir("testdir");
         tmpDir.deleteOnExit();
         File tmpFile = File.createTempFile("testfile", "", tmpDir);
         tmpFile.deleteOnExit();
@@ -366,6 +387,6 @@ public abstract class AbstractTestCase extends TestCase {
             return new File[] {tmpDir, tmpFile};
         } finally {
             fos.close();
-        }            
+        }
     }
 }
