@@ -66,7 +66,7 @@ public final class ZipTestCase extends AbstractTestCase {
         out.close();
 
         // Unarchive the same
-        List results = new ArrayList();
+        List<File> results = new ArrayList<File>();
 
         final InputStream is = new FileInputStream(output);
         ArchiveInputStream in = null;
@@ -94,9 +94,9 @@ public final class ZipTestCase extends AbstractTestCase {
         is.close();
 
         assertEquals(results.size(), 2);
-        File result = (File)results.get(0);
+        File result = results.get(0);
         assertEquals(file1.length(), result.length());
-        result = (File)results.get(1);
+        result = results.get(1);
         assertEquals(file2.length(), result.length());
     }
 
@@ -167,8 +167,8 @@ public final class ZipTestCase extends AbstractTestCase {
      */
     public void testListAllFilesWithNestedArchive() throws Exception {
         final File input = getFile("OSX_ArchiveWithNestedArchive.zip");
-       
-        List results = new ArrayList();
+
+        List<String> results = new ArrayList<String>();
 
         final InputStream is = new FileInputStream(input);
         ArchiveInputStream in = null;
@@ -192,7 +192,7 @@ public final class ZipTestCase extends AbstractTestCase {
             }
         }
         is.close();
-        
+
         results.contains("NestedArchiv.zip");
         results.contains("test1.xml");
         results.contains("test2.xml");
@@ -228,11 +228,9 @@ public final class ZipTestCase extends AbstractTestCase {
             if (zos != null) {
                 zos.close();
             }
-            if (archive != null) {
-                archive.delete();
-            }
-            tmp[1].delete();
-            tmp[0].delete();
+            tryHardToDelete(archive);
+            tryHardToDelete(tmp[1]);
+            rmdir(tmp[0]);
         }
     }
 
@@ -265,11 +263,9 @@ public final class ZipTestCase extends AbstractTestCase {
             if (zos != null) {
                 zos.close();
             }
-            if (archive != null) {
-                archive.delete();
-            }
-            tmp[1].delete();
-            tmp[0].delete();
+            tryHardToDelete(archive);
+            tryHardToDelete(tmp[1]);
+            rmdir(tmp[0]);
         }
     }
 
@@ -308,14 +304,12 @@ public final class ZipTestCase extends AbstractTestCase {
             if (zos != null) {
                 zos.close();
             }
-            if (archive != null) {
-                archive.delete();
-            }
+            tryHardToDelete(archive);
             if (fis != null) {
                 fis.close();
             }
-            tmp[1].delete();
-            tmp[0].delete();
+            tryHardToDelete(tmp[1]);
+            rmdir(tmp[0]);
         }
     }
 
@@ -356,14 +350,12 @@ public final class ZipTestCase extends AbstractTestCase {
             if (zos != null) {
                 zos.close();
             }
-            if (archive != null) {
-                archive.delete();
-            }
+            tryHardToDelete(archive);
             if (fis != null) {
                 fis.close();
             }
-            tmp[1].delete();
-            tmp[0].delete();
+            tryHardToDelete(tmp[1]);
+            rmdir(tmp[0]);
         }
     }
 }
