@@ -20,9 +20,13 @@ package org.apache.commons.compress.archivers.tar;
 
 import junit.framework.TestCase;
 
+import org.apache.commons.compress.archivers.zip.ZipEncoding;
+import org.apache.commons.compress.archivers.zip.ZipEncodingHelper;
+import org.apache.commons.compress.utils.CharsetNames;
+
 public class TarUtilsTest extends TestCase {
 
-    
+
     public void testName(){
         byte [] buff = new byte[20];
         String sb1 = "abcdefghijklmnopqrstuvwxyz";
@@ -38,13 +42,13 @@ public class TarUtilsTest extends TestCase {
         sb2 = TarUtils.parseName(buff, 1, buff.length-1);
         assertEquals(sb1, sb2);
     }
-    
+
     public void testParseOctal() throws Exception{
         long value; 
         byte [] buffer;
         final long MAX_OCTAL  = 077777777777L; // Allowed 11 digits
         final String maxOctal = "77777777777 "; // Maximum valid octal
-        buffer = maxOctal.getBytes("UTF-8");
+        buffer = maxOctal.getBytes(CharsetNames.UTF_8);
         value = TarUtils.parseOctal(buffer,0, buffer.length);
         assertEquals(MAX_OCTAL, value);
         buffer[buffer.length-1]=0;
@@ -52,10 +56,10 @@ public class TarUtilsTest extends TestCase {
         assertEquals(MAX_OCTAL, value);
         buffer=new byte[]{0,0};
         value = TarUtils.parseOctal(buffer,0, buffer.length);
-        assertEquals(0, value);        
+        assertEquals(0, value);
         buffer=new byte[]{0,' '};
         value = TarUtils.parseOctal(buffer,0, buffer.length);
-        assertEquals(0, value);        
+        assertEquals(0, value);
     }
 
     public void testParseOctalInvalid() throws Exception{
@@ -72,37 +76,31 @@ public class TarUtilsTest extends TestCase {
             fail("Expected IllegalArgumentException - should be at least 2 bytes long");
         } catch (IllegalArgumentException expected) {
         }
-        buffer=new byte[]{0,0,' '}; // not all NULs
-        try {
-            TarUtils.parseOctal(buffer,0, buffer.length);
-            fail("Expected IllegalArgumentException - not all NULs");
-        } catch (IllegalArgumentException expected) {
-        }
         buffer=new byte[]{' ',0,0,0}; // not all NULs
         try {
             TarUtils.parseOctal(buffer,0, buffer.length);
             fail("Expected IllegalArgumentException - not all NULs");
         } catch (IllegalArgumentException expected) {
         }
-        buffer = "abcdef ".getBytes("UTF-8"); // Invalid input
+        buffer = "abcdef ".getBytes(CharsetNames.UTF_8); // Invalid input
         try {
             TarUtils.parseOctal(buffer,0, buffer.length);
             fail("Expected IllegalArgumentException");
         } catch (IllegalArgumentException expected) {
         }
-        buffer = "77777777777".getBytes("UTF-8"); // Invalid input - no trailer
+        buffer = "77777777777".getBytes(CharsetNames.UTF_8); // Invalid input - no trailer
         try {
             TarUtils.parseOctal(buffer,0, buffer.length);
             fail("Expected IllegalArgumentException - no trailer");
         } catch (IllegalArgumentException expected) {
         }
-        buffer = " 0 07 ".getBytes("UTF-8"); // Invalid - embedded space
+        buffer = " 0 07 ".getBytes(CharsetNames.UTF_8); // Invalid - embedded space
         try {
             TarUtils.parseOctal(buffer,0, buffer.length);
             fail("Expected IllegalArgumentException - embedded space");
         } catch (IllegalArgumentException expected) {
         }
-        buffer = " 0\00007 ".getBytes("UTF-8"); // Invalid - embedded NUL
+        buffer = " 0\00007 ".getBytes(CharsetNames.UTF_8); // Invalid - embedded NUL
         try {
             TarUtils.parseOctal(buffer,0, buffer.length);
             fail("Expected IllegalArgumentException - embedded NUL");
@@ -110,22 +108,56 @@ public class TarUtilsTest extends TestCase {
         }
     }
 
-    private void checkRoundTripOctal(final long value) {
-        byte [] buffer = new byte[12];
+    private void checkRoundTripOctal(final long value, final int bufsize) {
+        byte [] buffer = new byte[bufsize];
         long parseValue;
         TarUtils.formatLongOctalBytes(value, buffer, 0, buffer.length);
         parseValue = TarUtils.parseOctal(buffer,0, buffer.length);
         assertEquals(value,parseValue);
     }
-    
+
+    private void checkRoundTripOctal(final long value) {
+        checkRoundTripOctal(value, TarConstants.SIZELEN);
+    }
+
     public void testRoundTripOctal() {
         checkRoundTripOctal(0);
         checkRoundTripOctal(1);
 //        checkRoundTripOctal(-1); // TODO What should this do?
-        checkRoundTripOctal(077777777777L);
+        checkRoundTripOctal(TarConstants.MAXSIZE);
 //        checkRoundTripOctal(0100000000000L); // TODO What should this do?
+
+        checkRoundTripOctal(0, TarConstants.UIDLEN);
+        checkRoundTripOctal(1, TarConstants.UIDLEN);
+        checkRoundTripOctal(TarConstants.MAXID, 8);
     }
-    
+
+    private void checkRoundTripOctalOrBinary(final long value, final int bufsize) {
+        byte [] buffer = new byte[bufsize];
+        long parseValue;
+        TarUtils.formatLongOctalOrBinaryBytes(value, buffer, 0, buffer.length);
+        parseValue = TarUtils.parseOctalOrBinary(buffer,0, buffer.length);
+        assertEquals(value,parseValue);
+    }
+
+    public void testRoundTripOctalOrBinary8() {
+        testRoundTripOctalOrBinary(8);
+    }
+
+    public void testRoundTripOctalOrBinary12() {
+        testRoundTripOctalOrBinary(12);
+        checkRoundTripOctalOrBinary(Long.MAX_VALUE, 12);
+        checkRoundTripOctalOrBinary(Long.MIN_VALUE + 1, 12);
+    }
+
+    private void testRoundTripOctalOrBinary(int length) {
+        checkRoundTripOctalOrBinary(0, length);
+        checkRoundTripOctalOrBinary(1, length);
+        checkRoundTripOctalOrBinary(TarConstants.MAXSIZE, length); // will need binary format
+        checkRoundTripOctalOrBinary(-1, length); // will need binary format
+        checkRoundTripOctalOrBinary(0xff00000000000001l, length);
+    }
+
     // Check correct trailing bytes are generated
     public void testTrailers() {
         byte [] buffer = new byte[12];
@@ -141,31 +173,38 @@ public class TarUtilsTest extends TestCase {
         assertEquals(0  , buffer[buffer.length-2]);
         assertEquals('3', buffer[buffer.length-3]); // end of number
     }
-    
-    public void testNegative() {
+
+    public void testNegative() throws Exception {
         byte [] buffer = new byte[22];
         TarUtils.formatUnsignedOctalString(-1, buffer, 0, buffer.length);
-        assertEquals("1777777777777777777777", new String(buffer));
+        assertEquals("1777777777777777777777", new String(buffer, CharsetNames.UTF_8));
     }
 
-    public void testOverflow() {
+    public void testOverflow() throws Exception {
         byte [] buffer = new byte[8-1]; // a lot of the numbers have 8-byte buffers (nul term)
         TarUtils.formatUnsignedOctalString(07777777L, buffer, 0, buffer.length);
-        assertEquals("7777777", new String(buffer));        
+        assertEquals("7777777", new String(buffer, CharsetNames.UTF_8));
         try {
             TarUtils.formatUnsignedOctalString(017777777L, buffer, 0, buffer.length);
             fail("Should have cause IllegalArgumentException");
         } catch (IllegalArgumentException expected) {
         }
     }
-    
+
     public void testRoundTripNames(){
         checkName("");
         checkName("The quick brown fox\n");
         checkName("\177");
         // checkName("\0"); // does not work, because NUL is ignored
+    }
+
+    public void testRoundEncoding() throws Exception {
         // COMPRESS-114
-        checkName("0302-0601-3F06W220ZBLALALACANDC04060302MOE.model");
+        ZipEncoding enc = ZipEncodingHelper.getZipEncoding(CharsetNames.ISO_8859_1);
+        String s = "0302-0601-3\u00b1\u00b1\u00b1F06\u00b1W220\u00b1ZB\u00b1LALALA\u00b1\u00b1\u00b1\u00b1\u00b1\u00b1\u00b1\u00b1\u00b1\u00b1CAN\u00b1\u00b1DC\u00b1\u00b1\u00b104\u00b1060302\u00b1MOE.model";
+        byte buff[] = new byte[100];
+        int len = TarUtils.formatNameBytes(s, buff, 0, buff.length, enc);
+        assertEquals(s, TarUtils.parseName(buff, 0, len, enc));
     }
 
     private void checkName(String string) {
@@ -173,4 +212,31 @@ public class TarUtilsTest extends TestCase {
         int len = TarUtils.formatNameBytes(string, buff, 0, buff.length);
         assertEquals(string, TarUtils.parseName(buff, 0, len));
     }
+
+    public void testReadNegativeBinary8Byte() {
+        byte[] b = new byte[] {
+            (byte) 0xff, (byte) 0xff, (byte) 0xff, (byte) 0xff,
+            (byte) 0xff, (byte) 0xff, (byte) 0xf1, (byte) 0xef,
+        };
+        assertEquals(-3601l, TarUtils.parseOctalOrBinary(b, 0, 8));
+    }
+
+    public void testReadNegativeBinary12Byte() {
+        byte[] b = new byte[] {
+            (byte) 0xff, (byte) 0xff, (byte) 0xff, (byte) 0xff,
+            (byte) 0xff, (byte) 0xff, (byte) 0xff, (byte) 0xff,
+            (byte) 0xff, (byte) 0xff, (byte) 0xf1, (byte) 0xef,
+        };
+        assertEquals(-3601l, TarUtils.parseOctalOrBinary(b, 0, 12));
+    }
+
+
+    public void testWriteNegativeBinary8Byte() {
+        byte[] b = new byte[] {
+            (byte) 0xff, (byte) 0xff, (byte) 0xff, (byte) 0xff,
+            (byte) 0xff, (byte) 0xff, (byte) 0xf1, (byte) 0xef,
+        };
+        assertEquals(-3601l, TarUtils.parseOctalOrBinary(b, 0, 8));
+    }
+
 }
